@@ -36,7 +36,6 @@ import org.apache.cxf.BusFactory;
 import org.apache.cxf.connector.CXFConnectionFactory;
 import org.apache.hello_world_soap_http.Greeter;
 
-import org.easymock.EasyMock;
 import org.junit.Before;
 import org.junit.Test;
 
@@ -46,7 +45,8 @@ import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertSame;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
-
+import static org.mockito.Mockito.mock;
+import static org.mockito.Mockito.when;
 
 public class ManagedConnectionFactoryImplTest {
 
@@ -171,14 +171,10 @@ public class ManagedConnectionFactoryImplTest {
         BusFactory bf = BusFactory.newInstance();
         Bus bus = bf.createBus();
         BusFactory.setDefaultBus(bus);
-        ManagedConnectionFactoryImpl factory = EasyMock.createMock(ManagedConnectionFactoryImpl.class);
-        factory.getBus();
+        ManagedConnectionFactoryImpl factory = mock(ManagedConnectionFactoryImpl.class);
+        when(factory.getBus()).thenReturn(bus);
         // In ManagedConnectionImpl:
         // one for getCXFServiceFromBus , another for createInvocationHandler
-        EasyMock.expectLastCall().andReturn(bus).times(4);
-
-        EasyMock.replay(factory);
-
 
         ManagedConnectionImpl mc1 = new ManagedConnectionImpl(factory, cri, subj);
         Object connection = mc1.getConnection(subj, cri);
@@ -230,7 +226,7 @@ public class ManagedConnectionFactoryImplTest {
     @Test
     public void testCreateConnectionFactoryCM() throws Exception {
         ManagedConnectionFactoryImplTester mcit = new ManagedConnectionFactoryImplTester();
-        ConnectionManager connManager = EasyMock.createMock(ConnectionManager.class);
+        ConnectionManager connManager = mock(ConnectionManager.class);
         assertTrue("We get a CF back",
                    mcit.createConnectionFactory(connManager) instanceof CXFConnectionFactory);
         assertEquals("init was called once", 1, mcit.initCalledCount);
