@@ -151,9 +151,9 @@ public class AsyncHTTPConduitFactory implements HTTPConduitFactory {
 
     private int ioThreadCount = IOReactorConfig.DEFAULT.getIoThreadCount();
     private long selectInterval = IOReactorConfig.DEFAULT.getSelectInterval().toMilliseconds();
-    private TimeValue soLinger = IOReactorConfig.DEFAULT.getSoLinger();
+    private int soLinger = IOReactorConfig.DEFAULT.getSoLinger().toMillisecondsIntBound();
     private int soTimeout = IOReactorConfig.DEFAULT.getSoTimeout().toMillisecondsIntBound();
-    private boolean soKeepalive = IOReactorConfig.DEFAULT.isSoKeepAlive();
+    private boolean soKeepalive = IOReactorConfig.DEFAULT.isSoKeepalive();
     private boolean tcpNoDelay = true;
 
 
@@ -223,13 +223,11 @@ public class AsyncHTTPConduitFactory implements HTTPConduitFactory {
         selectInterval = getInt(s.get(SELECT_INTERVAL), 1000);
         changed |= l != selectInterval;
 
-        i = soLinger.toSecondsIntBound();
-        // SO_LINGER timeout is set in seconds 
-        soLinger = TimeValue.ofSeconds(getInt(s.get(SO_LINGER), -1));
-        changed |= i != soLinger.toSecondsIntBound();
+        i = soLinger;
+        soLinger = getInt(s.get(SO_LINGER), -1);
+        changed |= i != soLinger;
 
         i = soTimeout;
-        // SO_TIMEOUT timeout is set in milliseconds
         soTimeout = getInt(s.get(SO_TIMEOUT), 0);
         changed |= i != soTimeout;
 
@@ -340,7 +338,7 @@ public class AsyncHTTPConduitFactory implements HTTPConduitFactory {
         final IOReactorConfig config = IOReactorConfig.custom()
             .setIoThreadCount(ioThreadCount)
             .setSelectInterval(TimeValue.ofMilliseconds(selectInterval))
-            .setSoLinger(soLinger)
+            .setSoLinger(TimeValue.ofMilliseconds(soLinger))
             .setSoTimeout(Timeout.ofMilliseconds(soTimeout))
             .setSoKeepAlive(soKeepalive)
             .setTcpNoDelay(tcpNoDelay)
@@ -348,7 +346,7 @@ public class AsyncHTTPConduitFactory implements HTTPConduitFactory {
 
         
         final Lookup<TlsStrategy> tlsLookupStrategy = RegistryBuilder.<TlsStrategy>create()
-            .register("https", (tlsStrategy != null) ? tlsStrategy : DefaultClientTlsStrategy.getSystemDefault())
+            .register("https", (tlsStrategy != null) ? tlsStrategy : DefaultClientTlsStrategy.createSystemDefault())
             .build();
 
         final PoolingAsyncClientConnectionManager connectionManager = new PoolingAsyncClientConnectionManager(
