@@ -37,6 +37,10 @@ import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
+import java.util.concurrent.TimeUnit;
+import java.util.concurrent.locks.Condition;
+import java.util.concurrent.locks.Lock;
+import java.util.concurrent.locks.ReentrantLock;
 import java.util.logging.Level;
 
 import javax.net.ssl.HostnameVerifier;
@@ -54,7 +58,7 @@ import org.apache.cxf.message.MessageUtils;
 import org.apache.cxf.service.model.EndpointInfo;
 import org.apache.cxf.transport.http.Address;
 import org.apache.cxf.transport.http.Headers;
-import org.apache.cxf.transport.http.URLConnectionHTTPConduit;
+import org.apache.cxf.transport.http.HttpClientHTTPConduit;
 import org.apache.cxf.transport.https.HttpsURLConnectionInfo;
 import org.apache.cxf.transports.http.configuration.HTTPClientPolicy;
 import org.apache.cxf.version.Version;
@@ -75,7 +79,11 @@ import io.netty.handler.codec.http.HttpResponse;
 import io.netty.handler.ssl.SslHandler;
 
 
-public class NettyHttpConduit extends URLConnectionHTTPConduit implements BusLifeCycleListener {
+public class NettyHttpConduit extends HttpClientHTTPConduit implements BusLifeCycleListener {
+    /**
+     * Enable HTTP/2 support
+     */
+    public static final String ENABLE_HTTP2 = "org.apache.cxf.transports.http2.enabled";
     public static final String USE_ASYNC = "use.async.http.conduit";
     public static final String MAX_RESPONSE_CONTENT_LENGTH =
         "org.apache.cxf.transport.http.netty.maxResponseContentLength";
@@ -103,6 +111,7 @@ public class NettyHttpConduit extends URLConnectionHTTPConduit implements BusLif
 
     // Using Netty API directly
     protected void setupConnection(Message message, Address address, HTTPClientPolicy csPolicy) throws IOException {
+        propagateProtocolSettings(message, csPolicy);
 
         URI uri = address.getURI();
         boolean addressChanged = false;
@@ -173,7 +182,8 @@ public class NettyHttpConduit extends URLConnectionHTTPConduit implements BusLif
             message.put(Message.HTTP_REQUEST_METHOD, httpRequestMethod);
         }
         // setup a new NettyHttpClientRequest
-        final NettyHttpClientRequest request = new NettyHttpClientRequest(uri, httpRequestMethod);
+        final boolean enableHttp2 = "2.0".equals(csPolicy.getVersion());
+        final NettyHttpClientRequest request = new NettyHttpClientRequest(uri, httpRequestMethod, enableHttp2);
         final int ctimeout = determineConnectionTimeout(message, csPolicy);
         final int rtimeout = determineReceiveTimeout(message, csPolicy);
         final int maxResponseContentLength = determineMaxResponseContentLength(message);
@@ -185,6 +195,15 @@ public class NettyHttpConduit extends URLConnectionHTTPConduit implements BusLif
 
     }
 
+    private void propagateProtocolSettings(Message message, HTTPClientPolicy csPolicy) {
+        if (message != null) {
+            final Object o = message.getContextualProperty(ENABLE_HTTP2);
+            if (o != null) {
+                csPolicy.setVersion("2.0");
+            }
+        }
+    }
+
     protected OutputStream createOutputStream(Message message,
                                               boolean needToCacheRequest,
                                               boolean isChunking,
@@ -213,6 +232,7 @@ public class NettyHttpConduit extends URLConnectionHTTPConduit implements BusLif
 
     public class NettyWrappedOutputStream extends WrappedOutputStream {
         final HTTPClientPolicy csPolicy;
+        final boolean enableHttp2;
         NettyHttpClientRequest entity;
         volatile HttpResponse httpResponse;
         volatile Throwable exception;
@@ -222,6 +242,10 @@ public class NettyHttpConduit extends URLConnectionHTTPConduit implements BusLif
         ByteBuf outBuffer;
         OutputStream outputStream;
 
+        final Lock syncLock = new ReentrantLock();
+        final Condition connected = syncLock.newCondition();
+        final Condition responded = syncLock.newCondition();
+
         protected NettyWrappedOutputStream(Message message, boolean possibleRetransmit,
                                            boolean isChunking, int chunkThreshold, String conduitName, URI url) {
             super(message, possibleRetransmit, isChunking, chunkThreshold, conduitName, url);
@@ -230,6 +254,7 @@ public class NettyHttpConduit extends URLConnectionHTTPConduit implements BusLif
             int bufSize = csPolicy.getChunkLength() > 0 ? csPolicy.getChunkLength() : 16320;
             outBuffer = Unpooled.buffer(bufSize);
             outputStream = new ByteBufOutputStream(outBuffer);
+            enableHttp2 = "2.0".equals(csPolicy.getVersion());
         }
 
         protected ByteBuf getOutBuffer() {
@@ -237,65 +262,73 @@ public class NettyHttpConduit extends URLConnectionHTTPConduit implements BusLif
         }
 
 
-        protected synchronized HttpResponse getHttpResponse() throws IOException {
-            while (httpResponse == null) {
-                if (exception == null) { //already have an exception, skip waiting
-                    try {
-                        wait(entity.getReceiveTimeout());
-                    } catch (InterruptedException e) {
-                        throw new IOException(e);
+        protected HttpResponse getHttpResponse() throws IOException {
+            syncLock.lock();
+            try {
+                while (httpResponse == null) {
+                    if (exception == null) { //already have an exception, skip waiting
+                        try {
+                            responded.await(entity.getReceiveTimeout(), TimeUnit.MILLISECONDS);
+                        } catch (InterruptedException e) {
+                            throw new IOException(e);
+                        }
+                    }
+                    if (httpResponse == null) {
+    
+                        if (exception != null) {
+                            if (exception instanceof IOException) {
+                                throw (IOException)exception;
+                            }
+                            if (exception instanceof RuntimeException) {
+                                throw (RuntimeException)exception;
+                            }
+                            throw new IOException(exception);
+                        }
+    
+                        throw new SocketTimeoutException("Read Timeout");
                     }
                 }
-                if (httpResponse == null) {
-
-                    if (exception != null) {
-                        if (exception instanceof IOException) {
-                            throw (IOException)exception;
-                        }
-                        if (exception instanceof RuntimeException) {
-                            throw (RuntimeException)exception;
-                        }
-                        throw new IOException(exception);
-                    }
-
-                    throw new SocketTimeoutException("Read Timeout");
-                }
+                return httpResponse;
+            } finally {
+                syncLock.unlock();
             }
-            return httpResponse;
         }
 
         protected HttpContent getHttpResponseContent() throws IOException {
             return (HttpContent) getHttpResponse();
         }
 
-
-        protected synchronized Channel getChannel() throws IOException {
-            while (channel == null) {
-                if (exception == null) { //already have an exception, skip waiting
-                    try {
-                        // connection timeout
-                        wait(entity.getConnectionTimeout());
-                    } catch (InterruptedException e) {
-                        throw new IOException(e);
+        protected Channel getChannel() throws IOException {
+            syncLock.lock();
+            try {
+                while (channel == null) {
+                    if (exception == null) { //already have an exception, skip waiting
+                        try {
+                            // connection timeout
+                            connected.await(entity.getConnectionTimeout(), TimeUnit.MILLISECONDS);
+                        } catch (InterruptedException e) {
+                            throw new IOException(e);
+                        }
+                    }
+                    if (channel == null) {
+    
+                        if (exception != null) {
+                            if (exception instanceof IOException) {
+                                throw (IOException)exception;
+                            }
+                            if (exception instanceof RuntimeException) {
+                                throw (RuntimeException)exception;
+                            }
+                            throw new IOException(exception);
+                        }
+    
+                        throw new SocketTimeoutException("Connection Timeout");
                     }
                 }
-                if (channel == null) {
-
-                    if (exception != null) {
-                        if (exception instanceof IOException) {
-                            throw (IOException)exception;
-                        }
-                        if (exception instanceof RuntimeException) {
-                            throw (RuntimeException)exception;
-                        }
-                        throw new IOException(exception);
-                    }
-
-                    throw new SocketTimeoutException("Connection Timeout");
-                }
+                return channel;
+            } finally {
+                syncLock.unlock();
             }
-            return channel;
-
         }
 
 
@@ -354,22 +387,25 @@ public class NettyHttpConduit extends URLConnectionHTTPConduit implements BusLif
         }
 
         protected void connect(boolean output) {
+            final NettyHttpClientPipelineFactory handler; 
             if ("https".equals(url.getScheme())) {
                 TLSClientParameters clientParameters = findTLSClientParameters();
-                bootstrap.handler(new NettyHttpClientPipelineFactory(clientParameters, entity.getReceiveTimeout(),
-                    entity.getMaxResponseContentLength()));
+                handler = new NettyHttpClientPipelineFactory(clientParameters, entity.getReceiveTimeout(),
+                    entity.getMaxResponseContentLength(), enableHttp2);
             } else {
-                bootstrap.handler(new NettyHttpClientPipelineFactory(null, entity.getReceiveTimeout(),
-                    entity.getMaxResponseContentLength()));
+                handler = new NettyHttpClientPipelineFactory(null, entity.getReceiveTimeout(),
+                    entity.getMaxResponseContentLength(), enableHttp2);
             }
 
+            // Set handler
+            bootstrap.handler(handler);
+
             ChannelFuture connFuture =
                 bootstrap.connect(new InetSocketAddress(url.getHost(), url.getPort() != -1 ? url.getPort()
                                                             : "http".equals(url.getScheme()) ? 80 : 443));
 
             // Setup the call back on the NettyHttpClientRequest
             ChannelFutureListener listener = new ChannelFutureListener() {
-
                 @Override
                 public void operationComplete(ChannelFuture future) throws Exception {
                     if (future.isSuccess()) {
@@ -391,7 +427,6 @@ public class NettyHttpConduit extends URLConnectionHTTPConduit implements BusLif
 
             connFuture.addListener(listener);
 
-
             // setup the CxfResponseCallBack
             CxfResponseCallBack callBack = new CxfResponseCallBack() {
                 @Override
@@ -423,8 +458,17 @@ public class NettyHttpConduit extends URLConnectionHTTPConduit implements BusLif
                         @Override
                         public void operationComplete(ChannelFuture future) throws Exception {
                             if (future.isSuccess()) {
-                                ChannelFuture channelFuture = future.channel().writeAndFlush(entity);
-                                channelFuture.addListener(writeFailureListener);
+                                handler.whenReady(future.channel())
+                                    .addListener(new ChannelFutureListener() {
+                                        @Override
+                                        public void operationComplete(ChannelFuture future) throws Exception {
+                                            if (future.isSuccess()) {
+                                                ChannelFuture channelFuture = future.channel().writeAndFlush(entity);
+                                                channelFuture.addListener(writeFailureListener);
+                                            }
+                                        }
+                                    }
+                                );
                             }
                         }
                     });
@@ -655,37 +699,52 @@ public class NettyHttpConduit extends URLConnectionHTTPConduit implements BusLif
             //entity.getRequest().setChunked(true);
         }
 
-        protected synchronized void setHttpResponse(HttpResponse r) {
-            httpResponse = r;
-            if (isAsync) {
-                //got a response, need to start the response processing now
-                try {
-                    handleResponseOnWorkqueue(false, true);
-                    isAsync = false; // don't trigger another start on next block. :-)
-                } catch (Exception ex) {
-                    //ignore, we'll try again on the next consume;
+        protected void setHttpResponse(HttpResponse r) {
+            syncLock.lock();
+            try {
+                httpResponse = r;
+                if (isAsync) {
+                    //got a response, need to start the response processing now
+                    try {
+                        handleResponseOnWorkqueue(false, true);
+                        isAsync = false; // don't trigger another start on next block. :-)
+                    } catch (Exception ex) {
+                        //ignore, we'll try again on the next consume;
+                    }
                 }
+                responded.signalAll();
+            } finally {
+                syncLock.unlock();
             }
-            notifyAll();
         }
 
-        protected synchronized void setException(Throwable ex) {
-            exception = ex;
-            if (isAsync) {
-                //got a response, need to start the response processing now
-                try {
-                    handleResponseOnWorkqueue(false, true);
-                    isAsync = false; // don't trigger another start on next block. :-)
-                } catch (Exception ex2) {
-                    ex2.printStackTrace();
+        protected void setException(Throwable ex) {
+            syncLock.lock();
+            try {
+                exception = ex;
+                if (isAsync) {
+                    //got a response, need to start the response processing now
+                    try {
+                        handleResponseOnWorkqueue(false, true);
+                        isAsync = false; // don't trigger another start on next block. :-)
+                    } catch (Exception ex2) {
+                        ex2.printStackTrace();
+                    }
                 }
+                responded.signalAll();
+            } finally {
+                syncLock.unlock();
             }
-            notifyAll();
         }
 
-        protected synchronized void setChannel(Channel ch) {
-            channel = ch;
-            notifyAll();
+        protected void setChannel(Channel ch) {
+            syncLock.lock();
+            try {
+                channel = ch;
+                connected.signalAll();
+            } finally {
+                syncLock.unlock();
+            }
         }
     }
 
