@@ -32,13 +32,13 @@ import java.util.Map;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 
-import javax.servlet.ServletConfig;
-import javax.servlet.ServletContext;
-import javax.servlet.ServletRequest;
-import javax.servlet.http.HttpServletRequest;
-import javax.servlet.http.HttpServletResponse;
 import javax.xml.namespace.QName;
 
+import jakarta.servlet.ServletConfig;
+import jakarta.servlet.ServletContext;
+import jakarta.servlet.ServletRequest;
+import jakarta.servlet.http.HttpServletRequest;
+import jakarta.servlet.http.HttpServletResponse;
 import org.apache.cxf.Bus;
 import org.apache.cxf.attachment.AttachmentDataSource;
 import org.apache.cxf.common.logging.LogUtils;
@@ -63,7 +63,6 @@ import org.apache.cxf.message.ExchangeImpl;
 import org.apache.cxf.message.Message;
 import org.apache.cxf.message.MessageImpl;
 import org.apache.cxf.message.MessageUtils;
-import org.apache.cxf.phase.AbortedInvocationException;
 import org.apache.cxf.policy.PolicyDataEngine;
 import org.apache.cxf.security.SecurityContext;
 import org.apache.cxf.security.transport.TLSSessionInfo;
@@ -103,8 +102,8 @@ public abstract class AbstractHTTPDestination
     public static final String SERVICE_REDIRECTION = "http.service.redirection";
     private static final String HTTP_BASE_PATH = "http.base.path";
 
-    private static final String SSL_CIPHER_SUITE_ATTRIBUTE = "javax.servlet.request.cipher_suite";
-    private static final String SSL_PEER_CERT_CHAIN_ATTRIBUTE = "javax.servlet.request.X509Certificate";
+    private static final String SSL_CIPHER_SUITE_ATTRIBUTE = "jakarta.servlet.request.cipher_suite";
+    private static final String SSL_PEER_CERT_CHAIN_ATTRIBUTE = "jakarta.servlet.request.X509Certificate";
 
     private static final String DECODE_BASIC_AUTH_WITH_ISO8859 = "decode.basicauth.with.iso8859";
 
@@ -263,17 +262,8 @@ public abstract class AbstractHTTPDestination
         copyKnownRequestAttributes(req, inMessage);
 
         try {
-            try {
-                incomingObserver.onMessage(inMessage);
-                invokeComplete(context, req, resp, inMessage);
-            } catch (AbortedInvocationException ex) {
-                maybeResetAndCloseResponseOutputStream(resp);
-                if (ex.getRuntimeException() != null) {
-                    throw ex.getRuntimeException();
-                } else {
-                    throw ex;
-                }
-            } 
+            incomingObserver.onMessage(inMessage);
+            invokeComplete(context, req, resp, inMessage);
         } catch (SuspendedInvocationException ex) {
             if (ex.getRuntimeException() != null) {
                 throw ex.getRuntimeException();
@@ -328,7 +318,9 @@ public abstract class AbstractHTTPDestination
                     //Please note, exchange used to always get the "current" message
                     exchange.getInMessage().put(HTTP_REQUEST, new HttpServletRequestSnapshot(req));
                 }
-                super.cacheInput();
+                if (req.getContentLengthLong() != 0) {
+                    super.cacheInput();
+                }
             }
             private boolean isWSAddressingReplyToSpecified(Exchange ex) {
                 AddressingProperties map = ContextUtils.retrieveMAPs(ex.getInMessage(), false, false, false);
@@ -358,12 +350,26 @@ public abstract class AbstractHTTPDestination
         if (contextPath == null) {
             contextPath = "";
         }
-        String servletPath = req.getServletPath();
+        String servletPath = null;
+        try {
+            servletPath = req.getServletPath();
+        } catch (Exception ex) {
+            //could be AmbiguousURI per RFC
+            //and Jetty 12 can't handle it right now
+            servletPath = requestURI;
+        }
+        
         if (servletPath == null) {
             servletPath = "";
         }
         String contextServletPath = contextPath + servletPath;
-        String pathInfo = req.getPathInfo();
+        String pathInfo = null;
+        try {
+            pathInfo = req.getPathInfo();
+        } catch (Exception ex) {
+            //could be AmbiguousURI per RFC
+            //and Jetty 12 can't handle it right now
+        }
         if (pathInfo != null) {
             inMessage.put(Message.PATH_INFO, contextServletPath + pathInfo);
         } else {
@@ -402,10 +408,22 @@ public abstract class AbstractHTTPDestination
 
         SecurityContext httpSecurityContext = new SecurityContext() {
             public Principal getUserPrincipal() {
-                return req.getUserPrincipal();
+                //ensure we use req from the one saved in inMessage
+                //as this could be the cachedInput one in oneway and 
+                //ReplyTo is specified when ws-addressing is used
+                //which means we need to switch thread context
+                //and underlying transport might discard any data on the original stream
+                HttpServletRequest reqFromInMessage = (HttpServletRequest)exchange.getInMessage().get(HTTP_REQUEST);
+                return reqFromInMessage.getUserPrincipal();
             }
             public boolean isUserInRole(String role) {
-                return req.isUserInRole(role);
+                //ensure we use req from the one saved in inMessage
+                //as this could be the cachedInput one in oneway and 
+                //ReplyTo is specified when ws-addressing is used
+                //which means we need to switch thread context
+                //and underlying transport might discard any data on the original stream
+                HttpServletRequest reqFromInMessage = (HttpServletRequest)exchange.getInMessage().get(HTTP_REQUEST);
+                return reqFromInMessage.isUserInRole(role);
             }
         };
 
@@ -674,21 +692,6 @@ public abstract class AbstractHTTPDestination
         return responseStream;
     }
 
-    private void maybeResetAndCloseResponseOutputStream(HttpServletResponse response) throws IOException {
-        try {
-            // The Servlet API does not provide means to abort the response, the best
-            // we could do is reset buffers (only partial data is going to be sent) and close
-            // the connection.
-            if (!response.isCommitted()) {
-                response.setHeader(HttpHeaderHelper.CONNECTION, HttpHeaderHelper.CLOSE);
-                response.resetBuffer();
-                response.getOutputStream().close();
-            }
-        } catch (IllegalStateException ex) {
-            // response.getWriter() has already been called
-        }
-    }
-
     private void closeResponseOutputStream(HttpServletResponse response) throws IOException {
         try {
             response.getOutputStream().close();
